import { DynamoDB } from "@aws-sdk/client-dynamodb";
import { NodeHttpHandler } from "@aws-sdk/node-http-handler";
import Utils from "./../../API/IKUtils";
import sha1 from 'sha1';
//
import IKGlobals from './../../IKGlobals';
import IKDB from './../Prototype/IKDB';
//
const { Agent } = require("https");
//reusable connection
var DDB_CONN = null;
var DDB_CONN_HASH = null;
//
export default class IKDB_DDB extends IKDB {
  constructor(config, transaction) {
    super(config, transaction);
    this.tableName = config.tableName;
    this.region = config.region;
    //
    // const localConsole = (transaction ? transaction.logger : console);
    // if (config && this.tableName) { localConsole.debug(`Using table: ${this.tableName} on region: ${this.region}`); }
  }

  async connect() {
    const localConsole = (this.transaction ? this.transaction.logger : console);
    //Reusability logic
    if (!((!DDB_CONN && !DDB_CONN_HASH) || DDB_CONN_HASH != sha1(this.region))) {
      localConsole.debug("Reusing database connection");
      this.connection = DDB_CONN;
      return;
    }
    //setup db
    if (Utils.isOffline()) {
      localConsole.debug("Starting offline database connection");
      this.connection = new DynamoDB(IKGlobals.DynamoDBLocalConfig);
    } else {
      // localConsole.debug("Starting remote database connection");
      //initialize connection
      this.connection = new DynamoDB({ 
        region: this.region,
        requestHandler: new NodeHttpHandler({
          httpsAgent: new Agent({ keepAlive: false, maxSockets: 50, rejectUnauthorized: true }),
        })
      });
      //Reusable logic
      DDB_CONN = this.connection;
      DDB_CONN_HASH = sha1(this.region);
    }
  }
}
